from django.db import models
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.contrib.auth.models import User,Group
from django.dispatch import receiver
from django.db.models.signals import post_save, pre_save,pre_delete




class FileForSave(models.Model):
    docfile = models.FileField(upload_to='')




class EmployeeUser(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    note = models.CharField(max_length=50)
    create_by = models.ForeignKey(User, on_delete=models.CASCADE,related_name='+')
    def delete(self, *args, **kwargs):
        self.user.delete()
        return super(self.__class__, self).delete(*args, **kwargs)
    def __str__(self):
        return str(self.user)+' create_by '+str(self.create_by)

class POSManager(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    shop = models.CharField(max_length=150)
    def delete(self, *args, **kwargs):
        self.user.delete()
        return super(self.__class__, self).delete(*args, **kwargs)
    def __str__(self):
        return str(self.user)+' manage '+str(self.shop)


class Product(models.Model):
    product_id = models.CharField(max_length=50)
    name = models.CharField(max_length=50)
    description = models.CharField(max_length=50)
    category = models.CharField(max_length=50)
    price = models.DecimalField(max_digits=7,decimal_places=2)
    quantity = models.IntegerField()
    manager = models.ForeignKey(User, on_delete=models.CASCADE)
    class Meta:
        unique_together = (("product_id","manager"),)
    def __str__(self):
        return str(self.manager)+' create ' +str(self.product_id)+' '+self.name+' '+str(self.price)  
    def get_absolute_url(self):
        return reverse("show_product")


@receiver(pre_save, sender=User)
def update_username_from_email(sender, instance, **kwargs): 
    if instance.username.lower()==instance.first_name.lower() and instance.email!=None and instance.email!='' :
        user_email = instance.email
        username = user_email[:30]
        n = 1
        while User.objects.exclude(pk=instance.pk).filter(username=username).exists():
            n += 1
            username = user_email[:(29 - len(str(n)))] + '-' + str(n)
        instance.username = username

@receiver(post_save,sender=User)
def addGroupsManager(sender, instance, **kwargs):
    if instance.groups.exists() == False:
        group_manager = Group.objects.get(name='manager') 
        instance.groups.add(group_manager)


@receiver(post_save,sender=EmployeeUser)
def setGroupEmployeeDeleteGroupManager(sender,instance, **kwargs):
    group_employee = Group.objects.get(name='employee')
    instance.user.groups.add(group_employee)
    group_manager = Group.objects.get(name='manager')
    instance.user.groups.remove(group_manager)
    # edit เมื่อทำหน้าสมัครเอง
    if POSManager.objects.filter(user=instance.create_by).count()==0:
        POSManager(user=instance.create_by).save()