from django import urls
from django.shortcuts import redirect, render,get_object_or_404
import os
from django.urls import reverse
from .models import FileForSave, POSManager,Product,EmployeeUser
from .forms import FileForm,EmployeeForm,ProductForm,SendProductID
import pandas as pd
from django.contrib.auth.models import User
from django.db.models.signals import post_save
from django.views.generic.edit import UpdateView,DeleteView
from django.http import HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from .permission_page import pos_manager_only
from pos.models import Bill,BillList

@login_required
@pos_manager_only
def show_bills(req):
    context = dict()
    manager = POSManager.objects.get(user=req.user)
    context['bills'] = Bill.objects.filter(manager=manager).order_by('-created')
    return render(req,'show_bills.html',context)

@login_required
@pos_manager_only
def show_products(request):
    context = {}
    context['products'] = Product.objects.filter(manager=request.user).order_by('quantity')
    context['keepPID'] = SendProductID()
    return render(request,'show_product.html',context)


class BillDeleteView(DeleteView):
    template_name = 'delete_bill.html'
    def get_object(self):
        id_ = self.kwargs.get("id")
        return get_object_or_404(Bill, id=id_)
    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        success_url = self.get_success_url()
        print(success_url)
        self.object.delete()
        success_url = self.get_success_url()
        return HttpResponseRedirect(success_url)
    def get_success_url(self):
        return reverse('bills',kwargs=None)

@login_required
@pos_manager_only
def add_product(request):
    message = ''
    file_form = FileForm()
    product_form = ProductForm()  
    if request.method == 'POST':
        file_form = FileForm(request.POST, request.FILES)
        product_form = ProductForm(request.POST)
        if file_form.is_valid():
            newdoc = FileForSave(docfile=request.FILES['docfile'])
            newdoc.save()
            try:
                df = pd.read_excel (newdoc.docfile)
                for i in df.values:
                    i = i.tolist()
                    for j in i:
                        print(type(j),end=' ')
                    print()
                    try:
                        product = Product.objects.get(manager=request.user,product_id=i[0])
                        product.quantity+=i[4]
                        product.save()
                    except:
                        try:
                            instance_product = Product(
                            product_id=str(i[0],),
                            name=i[1],
                            description=i[2],
                            price=i[3],
                            quantity=i[4],
                            category=i[5],
                            manager=request.user)
                            instance_product.save()
                            print(post_save(sender=Product,instance=instance_product))
                            

                        except:
                            print('*--try level 3 error--* new_instance_product error')
                        print('*--try level 2 error--* no this product')
            except:
                print('*--try level 1 error--*')
                message = 'เกิดข้อผิดพลาด'
            allFile = FileForSave.objects.all()
            for i in allFile:
                f_path = i.docfile.path
                try:
                    os.remove(f_path)
                    print(f_path+' deleted')
                    i.delete()
                except:
                    print("can't delete "+f_path)  
            return redirect('show_product')
        elif product_form.is_valid():
            rp = request.POST
            instance_product = Product(
                            product_id=rp.get('product_id'),
                            name=rp.get('name'),
                            description=rp.get('description'),
                            price=rp.get('price'),
                            quantity=rp.get('quantity'),
                            category=rp.get('category'),
                            manager=request.user)

            instance_product.save()

            return redirect('show_product')  
        else:
            message = 'กรุณากรอกข้อมูลกรอกข้อมูล'

    context = {'upload_form': file_form, 'message': message}
    context['product_form']=product_form
 
    return render(request, 'add_product.html', context)


@login_required
@pos_manager_only
def manage_employee(request):
    sign_up_form = EmployeeForm()
    if request.method == 'POST':
        form = EmployeeForm(request.POST, request.FILES)
        if form.is_valid():
            user = User.objects.create(username=request.POST['username'])  
            user.set_password(raw_password=request.POST['password'])
            user.save()
            employee = EmployeeUser(user=user,create_by=request.user,note=request.POST['note'])
            employee.save()
            return redirect('manage_employee')
            
    employees = EmployeeUser.objects.filter(create_by=request.user).values()
    employees2 = []
    for i in employees:
        employees2.append({
            'username':User.objects.get(id=i['user_id']).username,
            'note':i['note'],
            'id':i['id']
            })
    return render(request,'manage_employee.html',{'sign_up_form':sign_up_form,'emps':employees2,'emp1':employees})


@login_required
@pos_manager_only
def edit_product(request):
    print('in function edit product ')
    if request.method == 'POST':
        print(request.POST)
        context = {}
        return render(request, 'edit_product.html' ,context)
    else : 
        return redirect('show_product')



class ProductUpdateView(UpdateView):
    model = Product
    form_class = ProductForm
    template_name = 'edit_product.html'
    def get_object(self):
        id_ = self.kwargs.get("id")
        return get_object_or_404(Product, id=id_)

    def form_valid(self, form):
        return super().form_valid(form)

class ProductDeleteView(DeleteView):
    template_name = "delete_product.html"
    def get_object(self):
        id_ = self.kwargs.get("id")
        print(get_object_or_404(Product, id=id_))
        return get_object_or_404(Product, id=id_)
    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        manager = User.objects.get(id = self.object.manager_id)
        success_url = self.get_success_url()
        if request.user == manager:
            success_url = self.get_success_url()
            print('delete ok')
            self.object.delete()
        else:
            print("you don't have permission")
        return HttpResponseRedirect(success_url)
    def get_success_url(self):
        return reverse('show_product',kwargs=None)

class EmployeeDeleteView(DeleteView):
    template_name = "delete_employee.html"
    def get_object(self):
        id_ = self.kwargs.get("id")
        return get_object_or_404(EmployeeUser, id=id_ )
    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.object.delete()

        success_url = self.get_success_url()

        return HttpResponseRedirect(success_url)
    def get_success_url(self):
        return reverse('manage_employee',kwargs=None)
